@@ -261,14 +261,16 @@ If COLLECTION is an `obarray', a TEST should be needed. See `obarray'."
                  ;; Normally file completion should not be handled here,
                  ;; but special cases like `find-file-at-point' do it.
                  ;; Handle here specially such cases.
-                 ((and (functionp collection) minibuffer-completing-file-name)
+                 ((and (functionp collection) (not (string= input ""))
+                       minibuffer-completing-file-name)
                   (cl-loop for f in (funcall collection input test t)
                            unless (member f '("./" "../"))
                            if (string-match helm--url-regexp input)
                            collect f
                            else
                            collect (concat (file-name-as-directory
-                                            (helm-basedir input)) f)))
+                                            (helm-basedir input))
+                                           f)))
                  ((functionp collection)
                   (funcall collection input test t))
                  ((and alistp (null test)) collection)
@@ -858,7 +860,7 @@ See documentation of `completing-read' and `all-completions' for details."
        (candidate-number-limit helm-ff-candidate-number-limit)
        nomark
        (alistp t)
-       (persistent-action 'helm-find-files-persistent-action)
+       (persistent-action-if 'helm-find-files-persistent-action-if)
        (persistent-help "Hit1 Expand Candidate, Hit2 or (C-u) Find file")
        (mode-line helm-read-file-name-mode-line-string))
   "Read a file name with helm completion.
@@ -892,7 +894,7 @@ Keys description:
 
 - ALISTP: Don't use `all-completions' in history (take effect only on history).
 
-- PERSISTENT-ACTION: a persistent action function.
+- PERSISTENT-ACTION-IF: a persistent if action function.
 
 - PERSISTENT-HELP: persistent help message.
 
@@ -918,7 +920,6 @@ Keys description:
          (helm-ff-auto-update-initial-value
           (and helm-ff-auto-update-initial-value
                (not (minibuffer-window-active-p (minibuffer-window)))))
-         helm-full-frame
          helm-follow-mode-persistent
          (helm-ff-fuzzy-matching
           (and fuzzy
@@ -956,7 +957,7 @@ Keys description:
              :candidates hist
              :nohighlight t
              :fuzzy-match fuzzy
-             :persistent-action persistent-action
+             :persistent-action-if persistent-action-if
              :persistent-help persistent-help
              :keymap cmap
              :nomark nomark
@@ -988,7 +989,7 @@ Keys description:
                            (helm-find-files-get-candidates must-match))))
              :filtered-candidate-transformer 'helm-ff-sort-candidates
              :filter-one-by-one 'helm-ff-filter-candidate-one-by-one
-             :persistent-action persistent-action
+             :persistent-action-if persistent-action-if
              :persistent-help persistent-help
              :volatile t
              :keymap cmap
@@ -1006,6 +1007,7 @@ Keys description:
                   :case-fold-search case-fold
                   :default default
                   :buffer buffer
+                  :full-frame nil
                   :preselect preselect)))
     (or
      (cond ((and result (stringp result)
@@ -1017,7 +1019,9 @@ Keys description:
             (if (listp default) (car default) default))
            ((and result (listp result))
             (mapcar #'expand-file-name result))
-           (t result))
+           ((and result (file-directory-p result))
+            (file-name-as-directory (expand-file-name result)))
+           (result (expand-file-name result)))
      (helm-mode--keyboard-quit))))
 
 (defun helm-mode--default-filename (fname dir initial)
