@@ -46,19 +46,22 @@ will not have anymore separators between candidates."
           (integer :tag "Max candidate offset"))
   :group 'helm-ring)
 
-(defcustom helm-register-max-offset 160
-  "Max size of string register entries before truncating."
-  :group 'helm-ring
-  :type  'integer)
-
 (defcustom helm-kill-ring-actions
-  '(("Yank" . helm-kill-ring-action-yank)
-    ("Delete" . helm-kill-ring-action-delete)
-    ("Append" . helm-kill-ring-append))
+  '(("Yank marked" . helm-kill-ring-action-yank)
+    ("Delete marked" . helm-kill-ring-action-delete))
   "List of actions for kill ring source."
   :group 'helm-ring
   :type '(alist :key-type string :value-type function))
 
+(defcustom helm-kill-ring-separator "\n"
+  "The separator used to separate marked candidates when yanking."
+  :group 'helm-ring
+  :type 'string)
+
+(defcustom helm-register-max-offset 160
+  "Max size of string register entries before truncating."
+  :group 'helm-ring
+  :type  'integer)
 
 ;;; Kill ring
 ;;
@@ -69,7 +72,6 @@ will not have anymore separators between candidates."
     (define-key map (kbd "M-y")     'helm-next-line)
     (define-key map (kbd "M-u")     'helm-previous-line)
     (define-key map (kbd "M-D")     'helm-kill-ring-delete)
-    (define-key map (kbd "C-M-w")   'helm-kill-ring-run-append)
     (define-key map (kbd "C-]")     'helm-kill-ring-toggle-truncated)
     (define-key map (kbd "C-c C-k") 'helm-kill-ring-kill-selection)
     map)
@@ -140,60 +142,70 @@ Same as `helm-kill-selection-and-quit' called with a prefix arg."
         (cl-return)
         (helm-next-line))))
 
-(defun helm-kill-ring-action-yank (str)
+(defun helm-kill-ring-action-yank (_str)
+  "Insert concatenated marked candidates in current-buffer.
+
+When two prefix args are given prompt to choose separator, otherwise
+use `helm-kill-ring-separator' as default."
+  (let ((marked (helm-marked-candidates))
+        (sep (if (equal helm-current-prefix-arg '(16))
+                 (read-string "Separator: ")
+               helm-kill-ring-separator)))
+    (helm-kill-ring-action-yank-1
+     (cl-loop for c in (butlast marked)
+              concat (concat c sep) into str
+              finally return (concat str (car (last marked)))))))
+  
+(defun helm-kill-ring-action-yank-1 (str)
   "Insert STR in `kill-ring' and set STR to the head.
 
 When called with a prefix arg, point and mark are exchanged without
 activating region.
 If this action is executed just after `yank',
 replace with STR as yanked string."
-  (with-helm-current-buffer
-    (unwind-protect
-         (progn
-           (setq kill-ring (delete str kill-ring))
-           ;; Adding a `delete-selection' property
-           ;; to `helm-kill-ring-action' is not working
-           ;; because `this-command' will be `helm-maybe-exit-minibuffer',
-           ;; so use this workaround (Issue #1520).
-           (when (and (region-active-p) delete-selection-mode)
-             (delete-region (region-beginning) (region-end)))
-           (if (not (eq (helm-attr 'last-command helm-source-kill-ring) 'yank))
-               (progn
-                 ;; Ensure mark is at beginning of inserted text.
-                 (push-mark)
-                 ;; When yanking in a helm minibuffer we need a small
-                 ;; delay to detect the mark in previous minibuffer. [1]
-                 (run-with-timer
-                  0.01 nil
-                  (lambda ()
-                    (insert-for-yank str)
-                    (when helm-current-prefix-arg
-                      ;; Same as exchange-point-and-mark but without
-                      ;; activating region.
-                      (goto-char (prog1 (mark t)
-                                   (set-marker (mark-marker)
-                                               (point)
-                                               helm-current-buffer)))))))
+  (let ((yank-fn (lambda (&optional before yank-pop)
+                   (insert-for-yank str)
+                   ;; Set the window start back where it was in
+                   ;; the yank command, if possible.
+                   (when yank-pop
+                     (set-window-start (selected-window) yank-window-start t))
+                   (when (or (equal helm-current-prefix-arg '(4)) before)
+                     ;; Same as exchange-point-and-mark but without
+                     ;; activating region.
+                     (goto-char (prog1 (mark t)
+                                  (set-marker (mark-marker)
+                                              (point)
+                                              helm-current-buffer)))))))
+    (with-helm-current-buffer
+      (unwind-protect
+           (progn
+             (setq kill-ring (delete str kill-ring))
+             ;; Adding a `delete-selection' property
+             ;; to `helm-kill-ring-action' is not working
+             ;; because `this-command' will be `helm-maybe-exit-minibuffer',
+             ;; so use this workaround (Issue #1520).
+             (when (and (region-active-p) delete-selection-mode)
+               (delete-region (region-beginning) (region-end)))
+             (if (not (eq (helm-attr 'last-command helm-source-kill-ring) 'yank))
+                 (progn
+                   ;; Ensure mark is at beginning of inserted text.
+                   (push-mark)
+                   ;; When yanking in a helm minibuffer we need a small
+                   ;; delay to detect the mark in previous minibuffer. [1]
+                   (run-with-timer 0.01 nil yank-fn))
                ;; from `yank-pop'
                (let ((inhibit-read-only t)
                      (before (< (point) (mark t))))
                  (if before
                      (funcall (or yank-undo-function 'delete-region) (point) (mark t))
-                     (funcall (or yank-undo-function 'delete-region) (mark t) (point)))
+                   (funcall (or yank-undo-function 'delete-region) (mark t) (point)))
                  (setq yank-undo-function nil)
                  (set-marker (mark-marker) (point) helm-current-buffer)
-                 ;; Same as [1]
-                 (run-with-timer 0.01 nil (lambda () (insert-for-yank str)))
-                 ;; Set the window start back where it was in the yank command,
-                 ;; if possible.
-                 (set-window-start (selected-window) yank-window-start t)
-                 (when before
-                   ;; This is like exchange-point-and-mark, but doesn't activate the mark.
-                   ;; It is cleaner to avoid activation, even though the command
-                   ;; loop would deactivate the mark because we inserted text.
-                   (goto-char (prog1 (mark t)
-                                (set-marker (mark-marker) (point) helm-current-buffer)))))))
-      (kill-new str))))
+                 ;; Same as [1] but use the same mark and point as in
+                 ;; the initial yank according to BEFORE even if no
+                 ;; prefix arg is given.
+                 (run-with-timer 0.01 nil yank-fn before 'pop))))
+        (kill-new str)))))
 (define-obsolete-function-alias 'helm-kill-ring-action 'helm-kill-ring-action-yank "2.4.0")
 
 (defun helm-kill-ring-action-delete (_candidate)
@@ -210,19 +222,6 @@ This is a command for `helm-kill-ring-map'."
   (with-helm-alive-p
     (helm-exit-and-execute-action 'helm-kill-ring-action-delete)))
 
-(defun helm-kill-ring-append (_candidate)
-  "Yank concatenated marked candidates."
-  (let ((marked (helm-marked-candidates)))
-    (helm-kill-ring-action-yank
-     (cl-loop for cand in marked
-              for sep = (if (string-match "\n\\'" cand) "" "\n")
-              concat (concat cand sep)))))
-
-(defun helm-kill-ring-run-append ()
-  "Yank concatenated marked candidates."
-  (interactive)
-  (with-helm-alive-p
-    (helm-exit-and-execute-action 'helm-kill-ring-append)))
 
 ;;;; <Mark ring>
 ;; DO NOT use these sources with other sources use
@@ -481,34 +480,57 @@ Define your macros with `f3' and `f4'.
 See (info \"(emacs) Keyboard Macros\") for detailed infos.
 This command is useful when used with persistent action."
   (interactive)
-  (helm :sources
-        (helm-build-sync-source "Kmacro"
-          :candidates (lambda ()
-                        (helm-fast-remove-dups
-                         (cons (kmacro-ring-head)
-                               kmacro-ring)
-                         :test 'equal))
-          :multiline t
-          :candidate-transformer
-          (lambda (candidates)
-            (cl-loop for c in candidates collect
-                     (propertize (help-key-description (car c) nil)
-                                 'helm-realvalue c)))
-          :persistent-help "Execute kmacro"
-          :help-message 'helm-kmacro-help-message
-          :action
-          (helm-make-actions
-           "Execute kmacro (`C-u <n>' to execute <n> times)"
-           (lambda (candidate)
-             (interactive)
-             ;; Move candidate on top of list for next use.
-             (setq kmacro-ring (delete candidate kmacro-ring))
-             (kmacro-push-ring)
-             (kmacro-split-ring-element candidate)
-             (kmacro-exec-ring-item
-              candidate helm-current-prefix-arg)))
-          :group 'helm-ring)
-        :buffer "*helm kmacro*"))
+  (let ((helm-quit-if-no-candidate
+         (lambda () (message "No kbd macro has been defined"))))
+    (helm :sources
+          (helm-build-sync-source "Kmacro"
+            :candidates (lambda ()
+                          (helm-fast-remove-dups
+                           (cons (kmacro-ring-head)
+                                 kmacro-ring)
+                           :test 'equal))
+            :multiline t
+            :candidate-transformer
+            (lambda (candidates)
+              (cl-loop for c in candidates collect
+                       (propertize (help-key-description (car c) nil)
+                                   'helm-realvalue c)))
+            :persistent-help "Execute kmacro"
+            :help-message 'helm-kmacro-help-message
+            :action
+            (helm-make-actions
+             "Execute kmacro (`C-u <n>' to execute <n> times)"
+             'helm-kbd-macro-execute
+             "Concat marked macros"
+             'helm-kbd-macro-concat-macros
+             "Delete marked macros"
+             'helm-kbd-macro-delete-macro)
+            :group 'helm-ring)
+          :buffer "*helm kmacro*")))
+
+(defun helm-kbd-macro-execute (candidate)
+  ;; Move candidate on top of list for next use.
+  (setq kmacro-ring (delete candidate kmacro-ring))
+  (kmacro-push-ring)
+  (kmacro-split-ring-element candidate)
+  (kmacro-exec-ring-item
+   candidate helm-current-prefix-arg))
+
+(defun helm-kbd-macro-concat-macros (_candidate)
+  (let ((mkd (helm-marked-candidates)))
+    (when (cdr mkd)
+      (kmacro-push-ring)
+      (setq last-kbd-macro
+            (mapconcat 'identity
+                       (cl-loop for km in mkd
+                                collect (car km))
+                       "")))))
+
+(defun helm-kbd-macro-delete-macro (_candidate)
+  (let ((mkd (helm-marked-candidates)))
+    (cl-loop for km in mkd
+             do (setq kmacro-ring (delete km kmacro-ring)))
+    (kmacro-pop-ring1)))
 
 (provide 'helm-ring)
 
